@@ -15,8 +15,8 @@ from pygments.lexers import PythonLexer, PythonTracebackLexer
 from pygments.style import Style
 from pygments.token import Keyword, Operator, Number, Name, Error, Comment, Token
 
-from prompt_toolkit import CommandLine
-from prompt_toolkit.code import Completion, Code
+from prompt_toolkit import CommandLine, AbortAction
+from prompt_toolkit.code import Completion, Code, ValidationError
 from prompt_toolkit.enums import LineMode
 from prompt_toolkit.history import FileHistory, History
 from prompt_toolkit.inputstream_handler import ViInputStreamHandler, EmacsInputStreamHandler, ViMode
@@ -29,6 +29,8 @@ import jedi
 import os
 import re
 import traceback
+import threading
+
 
 __all__ = ('PythonCommandLine', 'embed')
 
@@ -74,6 +76,8 @@ class PythonStyle(Style):
 
         # Grayed
         Token.Aborted:    '#aaaaaa',
+
+        Token.ValidationError:    'bg:#aa0000 #ffffff',
     }
 
 
@@ -85,10 +89,10 @@ class _PythonInputStreamHandlerMixin(object):
         """ Enable/Disable paste mode. """
         self._line.paste_mode = not self._line.paste_mode
         if self._line.paste_mode:
-            self._line.multiline = True
+            self._line.is_multiline = True
 
     def F7(self):
-        self._line.multiline = not self._line.multiline
+        self._line.is_multiline = not self._line.is_multiline
 
     def _auto_enable_multiline(self):
         """
@@ -104,12 +108,12 @@ class _PythonInputStreamHandlerMixin(object):
         # If we just typed a colon, or still have open brackets, always insert a real newline.
         if cursor_at_the_end and (self._line._colon_before_cursor() or
                                   self._line._has_unclosed_brackets()):
-            self._line.multiline = True
+            self._line.is_multiline = True
 
         # If the character before the cursor is a backslash (line continuation
         # char), insert a new line.
         elif cursor_at_the_end and (self._line.document.text_before_cursor[-1:] == '\\'):
-            self._line.multiline = True
+            self._line.is_multiline = True
 
     def tab(self):
         # When the 'tab' key is pressed with only whitespace character before the
@@ -124,48 +128,25 @@ class _PythonInputStreamHandlerMixin(object):
         if self._line.mode == LineMode.COMPLETE:
             self._line.complete_previous()
 
-class PythonViInputStreamHandler(_PythonInputStreamHandlerMixin, ViInputStreamHandler):
     def enter(self):
         self._auto_enable_multiline()
+        super(_PythonInputStreamHandlerMixin, self).enter()
 
-        if self._line.mode == LineMode.INCREMENTAL_SEARCH:
-            self._line.exit_isearch(restore_original_line=False)
 
-        elif self._line.multiline:
-            if self._vi_mode == ViMode.NAVIGATION:
-                # We are in VI-navigation mode after pressing `Meta`.
-                self._line.return_input()
-            else:
-                self._line.newline()
-        else:
-            # In single line input, always execute when pressing enter.
-            self._line.return_input()
+class PythonViInputStreamHandler(_PythonInputStreamHandlerMixin, ViInputStreamHandler):
+    pass
 
 
 class PythonEmacsInputStreamHandler(_PythonInputStreamHandlerMixin, EmacsInputStreamHandler):
-    def enter(self):
-        self._auto_enable_multiline()
-
-        if self._line.mode == LineMode.INCREMENTAL_SEARCH:
-            self._line.exit_isearch(restore_original_line=False)
-
-        elif self._line.multiline:
-            self._line.newline()
-
-        else:
-            # In single line input, always execute when pressing enter.
-            self._line.return_input()
-
-    def meta_enter(self):
-        self._line.return_input()
+    pass
 
 
 class PythonLine(Line):
     """
     Custom `Line` class with some helper functions.
     """
-    def reset(self):
-        super(PythonLine, self).reset()
+    def reset(self, *a, **kw):
+        super(PythonLine, self).reset(*a, **kw)
 
         #: Boolean `paste` flag. If True, don't insert whitespace after a
         #: newline.
@@ -173,10 +154,13 @@ class PythonLine(Line):
 
         #: Boolean `multiline` flag. If True, [Enter] will always insert a
         #: newline, and it is required to use [Meta+Enter] execute commands.
-        self.multiline = False
+        self.is_multiline = False
+
+        # Code signatures. (This is set asynchronously after a timeout.)
+        self.signatures = []
 
     def _text_changed(self):
-        self.multiline = '\n' in self.text
+        self.is_multiline = '\n' in self.text
 
     def _colon_before_cursor(self):
         return self.document.text_before_cursor[-1:] == ':'
@@ -299,6 +283,8 @@ class PythonPrompt(Prompt):
             result.extend(list(super(PythonPrompt, self).get_isearch_prompt()))
         elif self.line._arg_prompt_text:
             result.extend(list(super(PythonPrompt, self).get_arg_prompt()))
+        elif self.line.validation_error:
+            result.extend(self._get_error_tokens())
         else:
             result.extend(self._get_signature_tokens())
 
@@ -309,20 +295,9 @@ class PythonPrompt(Prompt):
         result = []
         append = result.append
         Signature = Token.Signature
-        script = self.code._get_jedi_script()
 
-        # Show signatures in help text.
-        if script:
-            try:
-                signatures = script.call_signatures()
-            except ValueError:
-                # e.g. in case of an invalid \x escape.
-                signatures = []
-        else:
-            signatures = []
-
-        if signatures:
-            sig = signatures[0] # Always take the first one.
+        if self.line.signatures:
+            sig = self.line.signatures[0] # Always take the first one.
 
             append((Token, '           '))
             append((Signature, sig.full_name))
@@ -340,6 +315,16 @@ class PythonPrompt(Prompt):
 
         return result
 
+    def _get_error_tokens(self):
+        if self.line.validation_error:
+            text = '%s (line=%s column=%s)' % (
+                    self.line.validation_error.message,
+                    self.line.validation_error.line + 1,
+                    self.line.validation_error.column + 1)
+            return [(Token.ValidationError, text)]
+        else:
+            return []
+
     def _get_toolbar_tokens(self):
         result = []
         append = result.append
@@ -384,12 +369,12 @@ class PythonPrompt(Prompt):
             else:
                 append((TB.Off, '[F6] Paste mode (off) '))
 
-            if self.line.multiline:
+            if self.line.is_multiline:
                 append((TB.On, '[F7] Multiline (on)  '))
             else:
                 append((TB.Off, '[F7] Multiline (off) '))
 
-            if self.line.multiline:
+            if self.line.is_multiline:
                 append((TB, '[Meta+Enter] Execute'))
             else:
                 append((TB, '                    '))
@@ -407,6 +392,13 @@ class PythonCode(Code):
         self._locals = locals
         super(PythonCode, self).__init__(document)
 
+    def validate(self):
+        """ Check input for Python syntax errors. """
+        try:
+            compile(self.text, '<input>', 'exec')
+        except SyntaxError as e:
+            raise ValidationError(e.lineno - 1, e.offset - 1, 'Syntax Error')
+
     def _get_tokens(self):
         """ Overwrite parent function, to change token types of non-matching
         brackets to Token.Error for highlighting. """
@@ -464,7 +456,8 @@ class PythonCode(Code):
 
 class PythonCommandLine(CommandLine):
     line_cls = PythonLine
-    style_cls = PythonStyle
+
+    enable_concurency = True
 
     def history_cls(self):
         if self.history_filename:
@@ -479,13 +472,15 @@ class PythonCommandLine(CommandLine):
         else:
             return PythonEmacsInputStreamHandler
 
-    def __init__(self, globals=None, locals=None, vi_mode=False, stdin=None, stdout=None, history_filename=None):
+    def __init__(self, globals=None, locals=None, vi_mode=False, stdin=None, stdout=None, history_filename=None, style_cls=PythonStyle):
         self.globals = globals or {}
         self.globals.update({ k: getattr(__builtins__, k) for k in dir(__builtins__) })
         self.locals = locals or {}
         self.history_filename = history_filename
+        self.style_cls = style_cls
 
         self.vi_mode = vi_mode
+        self.get_signatures_thread_running = False
 
         #: Incremeting integer counting the current statement.
         self.current_statement_index = 1
@@ -500,6 +495,42 @@ class PythonCommandLine(CommandLine):
 
         super(PythonCommandLine, self).__init__(stdin=stdin, stdout=stdout)
 
+    def on_input_timeout(self, code_obj):
+        """
+        When there is no input activity,
+        in another thread, get the signature of the current code.
+        """
+        # Never run multiple get-signature threads.
+        if self.get_signatures_thread_running:
+            return
+        self.get_signatures_thread_running = True
+
+        class GetSignatureThread(threading.Thread):
+            def run(t):
+                script = code_obj._get_jedi_script()
+
+                # Show signatures in help text.
+                if script:
+                    try:
+                        signatures = script.call_signatures()
+                    except ValueError:
+                        # e.g. in case of an invalid \x escape.
+                        signatures = []
+                else:
+                    signatures = []
+
+                self.get_signatures_thread_running = False
+
+                # Set signatures and redraw if the text didn't change in the
+                # meantime. Otherwise request new signatures.
+                if self._line.text == code_obj.text:
+                    self._line.signatures = signatures
+                    self.request_redraw()
+                else:
+                    self.on_input_timeout(self._line.create_code_obj())
+
+        GetSignatureThread().start()
+
     def start_repl(self):
         """
         Start the Read-Eval-Print Loop.
@@ -507,7 +538,9 @@ class PythonCommandLine(CommandLine):
         try:
             while True:
                 # Read
-                document = self.read_input()
+                document = self.read_input(
+                                on_abort=AbortAction.RETRY,
+                                on_exit=AbortAction.RAISE_EXCEPTION)
                 line = document.text
 
                 if line and not line.isspace():
@@ -552,7 +585,7 @@ class PythonCommandLine(CommandLine):
         print('\rKeyboardInterrupt')
 
 
-def embed(globals=None, locals=None, vi_mode=False, history_filename=None):
+def embed(globals=None, locals=None, vi_mode=False, history_filename=None, no_colors=False):
     """
     Call this to embed  Python shell at the current point in your program.
     It's similar to `IPython.embed` and `bpython.embed`. ::
@@ -562,5 +595,6 @@ def embed(globals=None, locals=None, vi_mode=False, history_filename=None):
 
     :param vi_mode: Boolean. Use Vi instead of Emacs key bindings.
     """
-    cli = PythonCommandLine(globals, locals, vi_mode=vi_mode, history_filename=history_filename)
+    cli = PythonCommandLine(globals, locals, vi_mode=vi_mode, history_filename=history_filename,
+            style_cls=(None if no_colors else PythonStyle))
     cli.start_repl()
