@@ -102,7 +102,11 @@ class InputStreamHandler(object):
         self._line.abort()
 
     def ctrl_d(self):
-        self._line.exit()
+        # When there is text, act as delete, otherwise call exit.
+        if self._line.text:
+            self._line.delete()
+        else:
+            self._line.exit()
 
     def ctrl_e(self):
         self._line.cursor_to_end_of_line()
@@ -237,16 +241,12 @@ class InputStreamHandler(object):
             # When enter pressed in isearch, quit isearch mode. (Multiline
             # isearch would be too complicated.)
             self._line.exit_isearch()
+
+        elif self._line.is_multiline:
+            self._line.newline()
         else:
             self._line.return_input()
 
-    def meta_enter(self): # ESC-enter should always accept. -> enter in VI
-                         # insert mode should insert a newline. For emacs not
-                         # sure yet.
-
-        # XXX: we never come here, I think...
-        self._line.newline()
-
 
 class EmacsInputStreamHandler(InputStreamHandler):
     """
@@ -357,7 +357,8 @@ class EmacsInputStreamHandler(InputStreamHandler):
         self._arg_count = _arg_count_append(self._arg_count, digit)
 
     def meta_enter(self):
-        pass
+        """ Alt + Enter. Should always accept input. """
+        self._line.return_input()
 
     def meta_backspace(self):
         """ Delete word backwards. """
@@ -558,8 +559,11 @@ class ViInputStreamHandler(InputStreamHandler):
     def enter(self):
         if self._line.mode == LineMode.INCREMENTAL_SEARCH:
             self._line.exit_isearch(restore_original_line=False)
+
         elif self._vi_mode == ViMode.NAVIGATION:
+            self._vi_mode = ViMode.INSERT
             self._line.return_input()
+
         else:
             super(ViInputStreamHandler, self).enter()
 
