@@ -6,7 +6,7 @@ from __future__ import unicode_literals
 
 from functools import wraps
 
-from .code import Code
+from .code import Code, ValidationError
 from .document import Document
 from .enums import IncrementalSearchDirection, LineMode
 from .prompt import Prompt
@@ -120,8 +120,8 @@ class _IncrementalSearchState(object):
         self.isearch_direction = IncrementalSearchDirection.FORWARD
         self.isearch_text = ''
 
-        self.original_cursor_position = original_cursor_position
         self.original_working_index = original_working_index
+        self.original_cursor_position = original_cursor_position
 
 
 class Line(object):
@@ -135,6 +135,11 @@ class Line(object):
     :attr prompt_cls: :class:`~prompt_toolkit.prompt.PromptBase` class.
     :attr history: :class:`~prompt_toolkit.history.History` instance.
     """
+    #: Boolean to indicate whether we should consider this line a multiline input.
+    #: If so, the `InputStreamHandler` can decide to insert newlines when pressing [Enter].
+    #: (Instead of accepting the input.)
+    is_multiline = False
+
     def __init__(self, renderer=None, code_cls=Code, prompt_cls=Prompt, history_cls=History):
         self.renderer = renderer
         self.code_cls = code_cls
@@ -153,9 +158,12 @@ class Line(object):
 
         self.reset()
 
-    def reset(self):
+    def reset(self, initial_value=''):
         self.mode = LineMode.NORMAL
-        self.cursor_position = 0
+        self.cursor_position = len(initial_value)
+
+        # `ValidationError` instance. (Will be set when the input is wrong.)
+        self.validation_error = None
 
         # State of Incremental-search
         self.isearch_state = None
@@ -172,7 +180,7 @@ class Line(object):
         #: Enter should process the current command and append to the real
         #: history.
         self._working_lines = self._history.strings[:]
-        self._working_lines.append('')
+        self._working_lines.append(initial_value)
         self.__working_index = len(self._working_lines) - 1
 
     ### <getters/setters>
@@ -185,10 +193,13 @@ class Line(object):
     def text(self, value):
         self._working_lines[self._working_index] = value
 
-        # Always quit autocomplete mode when the cursor position changes.
+        # Always quit autocomplete mode when the text changes.
         if self.mode == LineMode.COMPLETE:
             self.mode = LineMode.NORMAL
 
+        # Remove any validation errors.
+        self.validation_error = None
+
         self._text_changed()
 
     @property
@@ -197,12 +208,15 @@ class Line(object):
 
     @cursor_position.setter
     def cursor_position(self, value):
-        self.__cursor_position = value
+        self.__cursor_position = max(0, value)
 
         # Always quit autocomplete mode when the cursor position changes.
         if self.mode == LineMode.COMPLETE:
             self.mode = LineMode.NORMAL
 
+        # Remove any validation errors.
+        self.validation_error = None
+
     @property
     def _working_index(self):
         return self.__working_index
@@ -555,7 +569,10 @@ class Line(object):
 
         self.cursor_position += column - current_column
 
-    def _create_code_obj(self):
+    def create_code_obj(self):
+        """
+        Create `Code` instance from the current input.
+        """
         return self.code_cls(self.document)
 
     @_to_mode(LineMode.NORMAL)
@@ -563,7 +580,7 @@ class Line(object):
         """
         Get and show all completions
         """
-        results = list(self._create_code_obj().get_completions())
+        results = list(self.create_code_obj().get_completions())
 
         if results and self.renderer:
             self.renderer.render_completions(results)
@@ -573,7 +590,7 @@ class Line(object):
         """ Autocomplete.
         Returns true if there was a completion. """
         # On the first tab press, try to find one completion and complete.
-        result = self._create_code_obj().complete() # XXX: rename to get_common_completion()
+        result = self.create_code_obj().complete() # XXX: rename to get_common_completion()
         if result:
             self.text = self.text[:self.cursor_position] + result + self.text[self.cursor_position:]
             self.cursor_position += len(result)
@@ -622,7 +639,7 @@ class Line(object):
         Start completions. (Generate list of completions and initialize.)
         """
         # Generate list of all completions.
-        current_completions = list(self._create_code_obj().get_completions())
+        current_completions = list(self.create_code_obj().get_completions())
 
         if current_completions:
             self.complete_state = CompletionState(
@@ -668,7 +685,7 @@ class Line(object):
                 highlight_regions = [ ]
 
         else:
-            code = self._create_code_obj()
+            code = self.create_code_obj()
             highlight_regions = [ ]
 
         # Complete state
@@ -681,7 +698,8 @@ class Line(object):
         # Create prompt instance.
         return RenderContext(prompt, code, highlight_regions=highlight_regions,
                         complete_state=complete_state,
-                        abort=_abort, accept=_accept)
+                        abort=_abort, accept=_accept,
+                        validation_error=self.validation_error)
 
     @_to_mode(LineMode.NORMAL)
     def history_forward(self):
@@ -795,8 +813,6 @@ class Line(object):
         Abort input. (Probably Ctrl-C press)
         """
         render_context = self.get_render_context(_abort=True)
-
-        self.reset()
         raise Abort(render_context)
 
     @_to_mode(LineMode.NORMAL)
@@ -812,9 +828,21 @@ class Line(object):
         """
         Return the current line to the `CommandLine.read_input` call.
         """
-        code = self._create_code_obj()
+        code = self.create_code_obj()
         text = self.text
 
+        # Validate first. If not valid, set validation exception.
+        try:
+            code.validate()
+            self.validation_error = None
+        except ValidationError as e:
+            # Set cursor position (don't allow invalid values.)
+            cursor_position = self.document.translate_row_col_to_index(e.line, e.column)
+            self.cursor_position = min(max(0, cursor_position), len(self.text))
+
+            self.validation_error = e
+            return
+
         # Save at the tail of the history. (But don't if the last entry the
         # history is already the same.)
         if not len(self._history) or self._history[-1] != text:
@@ -874,13 +902,12 @@ class Line(object):
                 self.cursor_position += new_index
             else:
                 # No match, go back in the history.
-                for i in range(len(self._working_lines)):
-                    i2 = (self._working_index - i - 1) % len(self._working_lines)
-                    document = Document(self._working_lines[i2], len(self._working_lines[i2]))
+                for i in range(self._working_index - 1, -1, -1):
+                    document = Document(self._working_lines[i], len(self._working_lines[i]))
                     new_index = document.find_backwards(isearch_text)
                     if new_index is not None:
-                        self._working_index = i2
-                        self.cursor_position = len(self._working_lines[i2]) + new_index
+                        self._working_index = i
+                        self.cursor_position = len(self._working_lines[i]) + new_index
                         break
         else:
             # Try find at the current input.
@@ -890,12 +917,11 @@ class Line(object):
                 self.cursor_position += new_index
             else:
                 # No match, go forward in the history.
-                for i in range(len(self._working_lines)):
-                    i2 = (self._working_index + i + 1) % len(self._working_lines)
-                    document = Document(self._working_lines[i2], 0)
+                for i in range(self._working_index + 1, len(self._working_lines)):
+                    document = Document(self._working_lines[i], 0)
                     new_index = document.find(isearch_text, include_current_position=True)
                     if new_index is not None:
-                        self._working_index = i2
+                        self._working_index = i
                         self.cursor_position = new_index
                         break
 
